//========================================================================
// Project: none
// ----------------------------------------------------------------------
// Package: Motion controller for the mini-SSC	
// Authors: Vilas Kumar Chitrakaran (cvilas@ces.clemson.edu)
// Start Date: Sat May 18 06:46:12 edt 2002
// Compiler: GNU GCC 2.96
// Operating System: GNU/Linux 
// ----------------------------------------------------------------------  
// File: MiniSSCMotionController.cpp
// Implementation of the class MiniSSCMotionController.
// See MiniSSCMotionController.hpp for more details.
//========================================================================  
 
// ----- Project Includes -----
#include "MiniSSCMotionController.hpp"
#include "MiniSSCPort.hpp"

// ----- Standard libraries ----
#include <time.h>


//========================================================================
// MiniSSCMotionController::MiniSSCMotionController
//========================================================================

MiniSSCMotionController::MiniSSCMotionController( char *port, int baudRate, 
				 		  int motorRange, 
						  int addressOffset )
{
	if ( port == 0 )
	{
		cout << endl 
		     << "[MiniSSCMotionController::MiniSSCMotionController]: "
		     << "Serial port not specified." 
		     << endl;
		exit(1);
	}
	
	// set the SSC device parameters
	s_miniSSCPort = port;
	s_baudRate = baudRate;
	(motorRange == 180) ? (s_motorRange = 2) : (s_motorRange = 1);
	(addressOffset == 8) ? (s_addressOffset = 8) : (s_addressOffset = 0);

	// empty out the setpoints vector, and initialize
	// the first setpoint at ready position (all motors centered.)
	pthread_mutex_lock( &loopUpdateMutex );
	for ( int channel = 0; channel < 8; channel++ )
	{
		s_setpointsQueue[channel].erase( s_setpointsQueue[channel].begin(), 
		s_setpointsQueue[channel].end() );
		s_setpointsQueue[channel].push_back(127);
		d_previousPosition[channel] = 127;
	}
	s_setpointsQueue[8].erase( s_setpointsQueue[8].begin(), 
	s_setpointsQueue[8].end() );
	s_setpointsQueue[8].push_back(1);

	s_setpointsQueue[9].erase( s_setpointsQueue[9].begin(), 
	s_setpointsQueue[9].end() );
	s_setpointsQueue[9].push_back(0);
	pthread_mutex_unlock( &loopUpdateMutex );


	// set a decent speed control
	d_speedFactor = 1;

	// Start the MiniSSC control thread function
	d_exit = false;
	if( (pthread_create(&d_controlThread, NULL, s_controlLoop, NULL)) )
	{
		cout << endl 
		     << "[MiniSSCMotionController::MiniSSCMotionController]: "
		     << "Control thread creation failed."
		     << endl;
		exit(1);
	}
}


//========================================================================
// MiniSSCMotionController::~MiniSSCMotionController
//========================================================================

MiniSSCMotionController::~MiniSSCMotionController()
{
	// Finish execution of the control thread.
	if( (pthread_join(d_controlThread, &d_controlThreadResult)) != 0 )
	{
		cout << endl 
		     << "[MiniSSCMotionController::~MiniSSCMotionController]: "
		     << "Control thread join failed." 
		     << endl;
	}
}


//========================================================================
// MiniSSCMotionController::setSpeedFactor
//========================================================================

void MiniSSCMotionController::setSpeedFactor( int speed )
{
	// The speed of the motor varies between 5 steps to 50 steps
	// as the speed factor varies from 1 to 10.
	if ( (speed > 0) && (speed < 11) )
		d_speedFactor = speed;
}


//========================================================================
// MiniSSCMotionController::moveTo
//========================================================================

void MiniSSCMotionController::moveTo( int channel, double position )
{
	int motorPos;
	if ( (channel < 0) || (channel > 7) )
	{
		cout << endl 
		     << "[MiniSSCMotionController::moveTo]: "
		     << "Invalid channel specified. Setpoint ignored."
		     << endl;
	}
	else
	{
		motorPos = (int)(127 + position/(0.35 * s_motorRange));
			// motor moves 0.35 deg per pulse if range is +/- 90.

		if ( motorPos < 0 )
			motorPos = 0;
		if ( motorPos > 254 )
			motorPos = 254;

		// copy all setpoints for all except the specified channel
		// from the previous position
		pthread_mutex_lock( &loopUpdateMutex );
		for ( int motor = 0; motor < 8; motor++ )
		{
			if ( motor == channel)
			{
				s_setpointsQueue[motor].push_back( motorPos );
				d_previousPosition[motor] = motorPos;
			}
			else
			{
				s_setpointsQueue[motor].push_back(d_previousPosition[motor]);
			}
		}
		s_setpointsQueue[8].push_back( d_speedFactor );
		s_setpointsQueue[9].push_back( 0 );
		pthread_mutex_unlock( &loopUpdateMutex );
	}
}


void MiniSSCMotionController::moveTo( double *position )
{
	int motorPos;
	pthread_mutex_lock( &loopUpdateMutex );
	for ( int motor = 0; motor < 8; motor++ )
	{
		motorPos = (int)(127 + position[motor]/(0.35 * s_motorRange));
			// motor moves 0.35 deg per pulse if range is +/- 90.
		if ( motorPos < 0 )
			motorPos = 0;
		if ( motorPos > 254 )
			motorPos = 254;
		s_setpointsQueue[motor].push_back( motorPos );
		d_previousPosition[motor] = motorPos;
	}

	s_setpointsQueue[8].push_back( d_speedFactor );
	s_setpointsQueue[9].push_back( 0 );
	pthread_mutex_unlock( &loopUpdateMutex );
}


//========================================================================
// MiniSSCMotionController::stop
//========================================================================

void MiniSSCMotionController::stop( int delay )
{
	if (delay < 0)
		d_stopDelay = 0;
	else
		d_stopDelay = delay;
	
	// Write a new setpoint with stop.
	// copy all setpoints for all from the previous position
	pthread_mutex_lock( &loopUpdateMutex );
	for ( int motor = 0; motor < 8; motor++ )
	{
		s_setpointsQueue[motor].push_back( 
		s_setpointsQueue[motor]
		[ s_setpointsQueue[motor].size()- 1 ] );
	}
	s_setpointsQueue[8].push_back( d_speedFactor );
	s_setpointsQueue[9].push_back( d_stopDelay );
	pthread_mutex_unlock( &loopUpdateMutex );
}

//========================================================================
// MiniSSCMotionController::quit
//========================================================================

void MiniSSCMotionController::quit()
{
	d_exit = true;
}


//========================================================================
// static void *s_controlLoop
//========================================================================

static void *s_controlLoop(void *arg)
{
	MiniSSCPort controlPort(s_miniSSCPort, s_baudRate);
	// Control port.
	
	bool d_firstRun = true;
	// for pusposes of initialization

	struct timespec stopTime;
	// the stop timer.
	
	int currentPosition[8];
	// The current position of the motor.

	int nextPosition[8];
	// The previous position

	int positionDiff[8];
	// difference in position between current and desired.
	
	int numJumps[8];
	// The number of positions to jump for a movement. This
	// one sets the speed of motion.

	int maxJumps;
	// Maximum number of position jumps between a movement.
	// This sucker sets the max speed.

	int steps[8];
	// The next step

	int channel;
	// The SSC channel dealing with

	if (d_firstRun)
	{
		pthread_mutex_lock( &loopUpdateMutex );
		for(channel = 0; channel < 8; channel++)
		{
			currentPosition[channel] = s_setpointsQueue[channel][0];
		}
		d_firstRun = false;
		pthread_mutex_unlock( &loopUpdateMutex );
}

	while(1)
	{
		if( s_setpointsQueue[0].size() > 0 )
		{
			// Get the desired next position, and remove it 
			// from setpoint queue
			pthread_mutex_lock( &loopUpdateMutex );
			for(channel = 0; channel < 8; channel++)
			{
				nextPosition[channel] = s_setpointsQueue[channel][0];
				s_setpointsQueue[channel].erase( s_setpointsQueue[channel].begin() );
				
				// initialize number of jumps
				numJumps[channel] = 1;
			}
			// set max jumps, and erase speed factor for this
			// position from the setpoints queue..
			maxJumps = s_setpointsQueue[8][0] * (4 / s_motorRange);
			s_setpointsQueue[8].erase( s_setpointsQueue[8].begin() );

			// set delay after stop if any, and then erase it from setpoints queue..
			stopTime.tv_sec = s_setpointsQueue[9][0] / 1000;
			stopTime.tv_nsec = (s_setpointsQueue[9][0] % 1000) * 1000000;
			s_setpointsQueue[9].erase( s_setpointsQueue[9].begin() );
			pthread_mutex_unlock( &loopUpdateMutex );

			// Loop until we get to current position from the
			// previous position.

			do
			{
				for(channel = 0; channel < 8; channel++)
				{
					//find the difference between current and next step.
					positionDiff[channel] = nextPosition[channel] 
																- currentPosition[channel];
					
					// set steps to jump next time
					if ( abs(positionDiff[channel]) )
					{
						steps[channel] = (int)( positionDiff[channel] 
						/ abs(positionDiff[channel]) ) * numJumps[channel];
						
						if( abs(positionDiff[channel]) < maxJumps )
							numJumps[channel] = max((int)numJumps[channel] / 2, 1);
						else
							numJumps[channel] = min((int)numJumps[channel] * 2, maxJumps);
					}
					else
						steps[channel] = 0;

					// update current position
					currentPosition[channel] += steps[channel];
					if( currentPosition[channel] < 0 )
						currentPosition[channel] = 0;
					if( currentPosition[channel] > 255 )
						currentPosition[channel] = 255;

					// move motor by steps calculated above.
					controlPort.actuate(channel + s_addressOffset, currentPosition[channel]);
				}

			}while( abs(steps[0]) + abs(steps[1]) + abs(steps[2]) + abs(steps[3]) + 
							abs(steps[4]) + abs(steps[5]) + abs(steps[6]) + abs(steps[7]) > 0);
			
			//wait for the set delay time
			nanosleep(&stopTime, NULL);
		}
		else
		{
			if(d_exit)
				break;
		}
	}

	pthread_exit(EXIT_SUCCESS);
}
