//========================================================================
// Project: none
// ----------------------------------------------------------------------
// Package: Motion controller for the mini-SSC
// Authors: Vilas Kumar Chitrakaran (cvilas@ces.clemson.edu)
// Start Date: Sat May 18 06:46:12 edt 2002
// Compiler: GNU GCC 2.96
// Operating System: GNU/Linux 
// ---------------------------------------------------------------------- 
// File: MiniSSCMotionController.hpp
// Interface of the class MiniSSCPort.
//========================================================================

#ifndef INCLUDED_MiniSSCMotionController_hpp
#define INCLUDED_MiniSSCMotionController_hpp

// ------ Standard header files ------
#include <vector>
#include <pthread.h>
#include <semaphore.h>

using namespace std;
//========================================================================
// class MiniSSCMotionController
// ----------------------------------------------------------------------
// \brief
// Methods for controlling the motion of motors connected to MiniSSC-II.
//
// Only POSIX compliant functions are used.
//
// <b>Example Program:</b>
// \include MiniSSCMotionController.t.cpp
//========================================================================

static int s_baudRate;
	// Speed of communication between MiniSSC and the control
	// computer. this is either 2400 or 9600 baud depending on
	// the absence or the presence of the 'B' jumper onthe SSC.

static int s_addressOffset;
	// This sets the range of addresses for the channels on the SSC. It
	// is either (0-7) or (8-15), depending on the absence or the
	// presence of the 'I' jumper on the SSC.
	//  0	address range 0 - 7
	//  8	address range 8 - 15

static int s_motorRange;
	// The range of the motors - 90 degrees or 180 degrees, depending
	// on the absence or the presence of the 'R' jumper on the SSC

static vector<int> s_setpointsQueue[10];
	// A queue for setpoints of all 8 channels in SSC. The remaining 
	// two elements specify speed factor and stop time at the setpoint. 
	// Note that this vector holds the actual position to which the motors 
	// are driven and not offsets from center position.

static void *s_controlLoop(void *arg);
	// This function continuously reads the motion queue, updating
	// the positions of motors connected to MiniSSC continuously.

static char *s_miniSSCPort;
	// The serial port (name) to which Mini SSC is connected.

static bool d_exit;
	// Condition to exit the control thread

static pthread_mutex_t loopUpdateMutex;
	// protects erronous loop updates

// Class definition starts here

class MiniSSCMotionController
{
	public:
	MiniSSCMotionController( char *port = 0, int baudRate = 2400, 
				 int motorRange = 90, int addressOffset = 0);
		// The default constructor initializes the variables 
		// and connects to the MiniSSC device.
		//  port	 The serial port (name) MiniSSC is connected to.
		//  baudRate	 The speed of communication with MiniSSC.
		//		 Only 2400 and 9600 are valid baud rates
		//		 depending on the absence or the presence of 
		//		 the 'B' jumper on the SSC.
		//  motorRange	 The range of rotation of the motor. This is 
		//		 90 degrees or 180 degrees, depending on the 
		//		 absence or the presence of the 'R' jumper on the 
		//		 SSC.
		//  addressOffset It is either 0 or 8, depending on the absence or 
		// 		 the presence of the 'I' jumper on the SSC.

	~MiniSSCMotionController();
		// The destructor cleans up.

	void setSpeedFactor( int speed = 1 );
		// Sets the speed of motion between setpoints.
		// Set this to '1' for slowest motion and 
		// 10 for fast motion with smooth acceleration
		// and deceleration between setpoints. Invalid
		// values will cause the speedFactor to remain 
		// unchanged.

	void moveTo( int channel, double position );
		// Moves a motor to a particular location.
		//  channel	The motor/channel number on the
		//		MiniSSC that you want to drive.
		//		Valid values are 0 to 7.
		//  position	Position in the range of -90 to
		//		+90 degrees, relative to center position
		//		for the motor. Only +/- 45 degrees of 
		//		motion may be possible if the 'R'
		//		jumper is absent. In any case an error
		//		message is printed out whenever limits
		//		are exceeded.
 
	void moveTo( double *position );
		// A vector of 8 positions for moving all the motors
		// actuated by MiniSSC. Again, the values are in the
		// range of +/-90 degrees or +/-45 degrees.

	void stop( int delay );
		// Stop at a setpoint for a specified delay time.
		//  delay	Delay time in milliseconds.

	void quit();
		// Exit the control. Always call this function to exit
		// control program. The program exit is not immediate
		// and happens only when the setpoint queue goes empty
		// i.e., after the motors have reached their final 
		// setpoints.
		
	// ========== END OF INTERFACE ==========
	
	private:

	pthread_t d_controlThread;
		// The control thread identifier.
	
	void *d_controlThreadResult;
		// Return value from control thread function.
	
	int d_speedFactor;
		// The speed of motion is decided by this value.

	int d_stopDelay;
		// Stop period at a particular setpoint.
		
	int d_previousPosition[8];
		// A buffer to hold positions the last time
		// moveTo() was called.

};


#endif

