//========================================================================
// Project: none
// ----------------------------------------------------------------------
// Package: Mini SSC-II communication module
// Authors: Vilas Kumar Chitrakaran (cvilas@ces.clemson.edu)
// Start Date: Sat May 18 06:46:12 edt 2002
// Compiler: GNU GCC 2.96
// Operating System: GNU/Linux 
// ---------------------------------------------------------------------- 
// File: MiniSSCPort.hpp
// Interface of the class MiniSSCPort.
//========================================================================

#ifndef INCLUDED_MiniSSCPort_hpp
#define INCLUDED_MiniSSCPort_hpp

//#include <stdio.h>
//#include <stdlib.h>
//#include <string.h>
#include <iostream>

#include <termios.h>
//#include <sys/types.h>
//#include <sys/ioctl.h>
#include <fcntl.h>
//#include <sys/uio.h>
#include <unistd.h>

using namespace std;
//========================================================================
// class MiniSSCPort
// ----------------------------------------------------------------------
// \brief
// Functions required to communicate with a Mini SSC II controller from
// Scott Edwards Electronics, Inc. are implemented in this class. 
//
// Only POSIX compliant functions are used.
//
// <b>Example Program:</b>
// \include MiniSSCPort.t.cpp
//========================================================================

class MiniSSCPort 
{
  public:    
  
  //----- constructor and destructor -----
  
  MiniSSCPort (char *portName = 0, int baud = 2400 );              
  	// The contructor initializes all variables and sets up
		// the serial port for communication with MiniSSC device.
		//  portName		Serial port name (eg: /dev/ttyS0)
		//  baud		Baudrate, valid values are 2400 or 9600
		//			depending upon absence or presence of the
		//			'B' jumper on the SSC board.
		//--------------------------------------------------------------
		// See MiniSSC manual for jumper settings.
		//--------------------------------------------------------------	
  		
  ~MiniSSCPort();
  		// The default destructor cleans up.

  //----- Initialization functions -----
  
  inline void setPortName( char *portName );
  		// Allows the user to specify the serial port name.
  
  inline void setBaudRate( int baudRate );
		// Valid baud rates for the serial port are
		// 2400 and 9600

   void reset();
		// This function is called whenever the serial
		// port configuration is changed, or when it is
		// desired to reset the SSC II outputs.

  inline int isStatusOK();
  		// Returns 0 : No error.
  		//         1 : Invalid port accessed.
  		//	   2 : Communication error
  		//	   3 : Port not ready to accept data
  
  //----- Control functions -----
  
  inline int actuate(int channel, int position);
  		// The most important function in this class. This
  		// function drives the output channel (valid values
  		// are 0 to 7 or 8 to 15) to the desired position 
		// (valid values are 0 to 255)
 		// The range of rotation of the motor is either	90 or 
		// 180 depending on absence or presence	of the 'R' jumper 
		// on SSC board. The address range of the SSC is either
		// (0-7) or (8-15) depending on absence	or presence of 
		// the 'I' jumper on the SSC board.
		//  return	0 on success
		//		-1 on failure
 
  //---- public data members ----

  public:
  enum 	// This enum defines error states
  {
    e_ok = 0,		// Status OK
    e_invalidPort,// Invalid Port
    e_commError,		// Error in communicating with port
    e_notReady			// Port not ready for data yet.
  };


  // ========== END OF INTERFACE ==========

  //----- Private functions ----
  
  private: 
  inline void writeToPort (char *buffer, int length);
  		// Write from a buffer (of given length) of characters
  		// to the serial port. No check is made to verify the 
  		// validity of what is written out. This is the 
  		// responsibility of the user program

 //---- Private data members ----
  
  private: 
  int d_status;      // If not OK, some error has occurred
  int d_serialPort;  // Serial port file descriptor
  
  char *d_portName;
  		// the serial port to use.
  	
  int d_baudRate;
  		// baud rate of the serial port. (2400 or 9600)

};



//======================================================================== 
// Implementation of the MiniSSCPort follows
//========================================================================

//======================================================================== 
// MiniSSCPort::isStatusOK
//========================================================================

inline int MiniSSCPort::isStatusOK() 
{
  return (d_status);
}


//======================================================================== 
// MiniSSCPort::setPortName
//========================================================================

inline void MiniSSCPort::setPortName( char *portName ) 
{
  if( portName > 0 )
  		d_portName = portName;
  	
  	reset();
}


//======================================================================== 
// MiniSSCPort::setBaudRate
//========================================================================

inline void MiniSSCPort::setBaudRate( int baudRate )
{
  if( baudRate == 2400 || baudRate == 9600 )
  		d_baudRate = baudRate;
  	
  	reset();
}


//======================================================================== 
// MiniSSCPort::actuate
//========================================================================

inline int MiniSSCPort::actuate(int channel, int position)
{
	char buffer[3];
	buffer[0] = 0xFF;
	buffer[1] = (char)(channel);
	if ( ((channel >= 0) && (channel < 8)) || 
		((channel > 8) && (channel < 16)) )
	{
		if (position < 0)
			position = 0;
		if (position > 255)
			position = 255;

		buffer[2] = (char)position;
		writeToPort(buffer, 3);
	}
  	else
  	{
#ifdef INCLUDEDEBUGINFO
  		cout << "[MiniSSCPort::writeToSSC(channel, position)] : " 
  		     << "Invalid channel specified." << endl;
#endif
  		return -1;
  	}
	return 0;
}
 

//======================================================================== 
// MiniSSCPort::writeToPort
//========================================================================

inline void MiniSSCPort::writeToPort (char *buffer, int length) 
{
  if ( (write (d_serialPort, buffer, length)) != length )
  {
  	d_status = e_commError;

#ifdef INCLUDEDEBUGINFO
  	cout << "[MiniSSCPort::writeToPort(buffer, length)] : Communication error." 
  	     << endl;
#endif  		
  }
}

//======================================================================== 
// Inline definitions of MiniSSCPort class ends here.
//========================================================================

#endif
